//
// Cleversafe open-source code header - Version 1.2 - February 15, 2008
//
// Cleversafe Dispersed Storage(TM) is software for secure, private and
// reliable storage of the world's data using information dispersal.
//
// Copyright (C) 2005-2008 Cleversafe, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
// USA.
//
// Contact Information: Cleversafe, 224 North Desplaines Street, Suite 500 
// Chicago IL 60661
// email licensing@cleversafe.org
//
// END-OF-HEADER
//-----------------------
// @author: mmotwani
//
// Date: Oct 5, 2007
//---------------------

package org.cleversafe.serialization.raw.custom;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

import org.cleversafe.layer.grid.DataSlice;
import org.cleversafe.layer.grid.SliceName;
import org.cleversafe.serialization.GridSerializable;
import org.cleversafe.serialization.exceptions.EncodeDecodeException;

public class DataSliceSerialization extends CustomRawSerializationBase
{
   public DataSliceSerialization()
   {
      super(DataSlice.class);
   }

   @Override
   public GridSerializable read(DataInput in) throws EncodeDecodeException, IOException
   {
      DataSlice dataSlice = new DataSlice();

      // Read SliceName
      dataSlice.setSliceName(readObject(SliceName.class, in));

      // Read Transaction ID
      dataSlice.setTransactionId(in.readLong());

      // Read Data Byte Array
      dataSlice.setData(readByteArray(in));

      return dataSlice;
   }

   @Override
   public void write(DataOutput out, GridSerializable serializable) throws EncodeDecodeException,
         IOException
   {
      assert serializable instanceof DataSlice;
      DataSlice dataSlice = (DataSlice) serializable;

      if (dataSlice.getSliceName() == null)
      {
         throw new EncodeDecodeException("SliceName in a DataSlice must not be null");
      }

      // Write SliceName
      writeObject(out, dataSlice.getSliceName());

      // Write Transaction ID
      out.writeLong(dataSlice.getTransactionId());

      // Write Data Byte Array
      writeByteArray(out, dataSlice.getData());
   }

   @Override
   public int getMaxSerializedSize(GridSerializable serializable)
   {
      assert serializable instanceof DataSlice;
      DataSlice dataSlice = (DataSlice) serializable;

      int size = 0;
      size += calculateMaxSerializedSize(dataSlice.getSliceName());
      size += 8; // Transaction ID (Long)
      size += calculateMaxSerializedSize(dataSlice.getData());

      return size;
   }
}
