//
// Cleversafe open-source code header - Version 1.2 - February 15, 2008
//
// Cleversafe Dispersed Storage(TM) is software for secure, private and
// reliable storage of the world's data using information dispersal.
//
// Copyright (C) 2005-2008 Cleversafe, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
// USA.
//
// Contact Information: Cleversafe, 224 North Desplaines Street, Suite 500 
// Chicago IL 60661
// email licensing@cleversafe.org
//
// END-OF-HEADER
//-----------------------
// @author: mmotwani
//
// Date: Oct 7, 2007
//---------------------

package org.cleversafe.serialization.raw.custom;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.UUID;

import org.cleversafe.layer.protocol.PasswordAuthenticationRequest;
import org.cleversafe.serialization.GridSerializable;
import org.cleversafe.serialization.exceptions.EncodeDecodeException;

public class PasswordAuthenticationRequestSerialization extends BaseRequestSerialization
{
   public PasswordAuthenticationRequestSerialization()
   {
      super(PasswordAuthenticationRequest.class);
   }

   @Override
   public void write(DataOutput out, GridSerializable serializable) throws EncodeDecodeException,
         IOException
   {
      assert serializable instanceof PasswordAuthenticationRequest;
      PasswordAuthenticationRequest request = (PasswordAuthenticationRequest) serializable;

      if (request.getUsername() == null || request.getPassword() == null)
      {
         throw new EncodeDecodeException(
               "Username or Password in PasswordAuthenticationRequest must not be null");
      }

      writeString(out, request.getUsername());
      writeString(out, request.getPassword());
      writeString(out, request.getGridAccountID().toString());
   }

   @Override
   public GridSerializable read(DataInput in) throws EncodeDecodeException, IOException
   {
      // PasswordAuthenticationRequest( username, password, grid account UUID )
      return new PasswordAuthenticationRequest(readString(in), readString(in),
            UUID.fromString(readString(in)));
   }

   @Override
   public int getMaxSerializedSize(GridSerializable serializable)
   {
      assert serializable instanceof PasswordAuthenticationRequest;
      PasswordAuthenticationRequest request = (PasswordAuthenticationRequest) serializable;

      int size = super.getMaxSerializedSize(serializable);
      size += calculateMaxSerializedSize(request.getUsername());
      size += calculateMaxSerializedSize(request.getPassword());
      size += calculateMaxSerializedSize(request.getGridAccountID().toString());
      return size;
   }
}
