//
// Cleversafe open-source code header - Version 1.2 - February 15, 2008
//
// Cleversafe Dispersed Storage(TM) is software for secure, private and
// reliable storage of the world's data using information dispersal.
//
// Copyright (C) 2005-2008 Cleversafe, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
// USA.
//
// Contact Information: Cleversafe, 224 North Desplaines Street, Suite 500 
// Chicago IL 60661
// email licensing@cleversafe.org
//
// END-OF-HEADER
//-----------------------
// @author: mmotwani
//
// Date: Oct 4, 2007
//---------------------

package org.cleversafe.serialization.raw.custom;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

import org.cleversafe.layer.grid.SliceName;
import org.cleversafe.layer.grid.SourceName;
import org.cleversafe.serialization.GridSerializable;
import org.cleversafe.serialization.exceptions.EncodeDecodeException;

public class SliceNameSerialization extends CustomRawSerializationBase
{
   public SliceNameSerialization()
   {
      super(SliceName.class);
   }

   @Override
   public GridSerializable read(DataInput in) throws EncodeDecodeException, IOException
   {
      SliceName sliceName = new SliceName();

      // Read Slice Index
      sliceName.setSliceIndex(in.readInt());

      // Read SourceName
      sliceName.setSourceName(readObject(SourceName.class, in));

      return sliceName;
   }

   @Override
   public void write(DataOutput out, GridSerializable serializable) throws EncodeDecodeException,
         IOException
   {
      assert serializable instanceof SliceName;
      SliceName sliceName = (SliceName) serializable;

      if (sliceName.getSourceName() == null)
      {
         throw new EncodeDecodeException("SourceName in a SliceName must not be null");
      }

      // Write Slice Index
      out.writeInt(sliceName.getSliceIndex());

      // Write SourceName
      writeObject(out, sliceName.getSourceName());
   }

   @Override
   public int getMaxSerializedSize(GridSerializable serializable)
   {
      assert serializable instanceof SliceName;
      SliceName sliceName = (SliceName) serializable;

      int size = 4; // Slice Index
      size += calculateMaxSerializedSize(sliceName.getSourceName());

      return size;
   }
}
